package dk.hansen.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.log4j.Logger;

import dk.hansen.dao.bean.DVD;
import dk.hansen.dao.bean.Key;

public class DVDDAO extends DataBase implements DVDDAOIF {

  private static Logger logger = Logger.getLogger(DVDDAO.class.getName());

  public void createDVD(String id, String title, String directorId) throws DAOException {
    if (findDVD(id) != null)
      throw new DAOException("Id " + id + " is already used");
    Connection conn = null;
    PreparedStatement pstmt = null;
    error = null;

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.createDVD, id=" + id);
      // Prepare a statement to insert a record
      String sql = "INSERT INTO dvd (id, title, director) VALUES(?, ?, ?)";
      pstmt = conn.prepareStatement(sql);
      pstmt.setString(1, id);
      pstmt.setString(2, title);
      pstmt.setString(3, directorId);
      pstmt.executeUpdate();
    } catch (SQLException e) {
      error = e;
      message = "Create failed";
    }

    closePrep(pstmt);
    closeConnection(conn);
    checkOK();
  }

  public DVD findDVD(String id) throws DAOException {
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet result = null;
    error = null;
    DVD dvd = null;

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.getDVD, id=" + id);
      // Prepare a statement to insert a record
      String sql = "SELECT * FROM dvd WHERE id=?";
      pstmt = conn.prepareStatement(sql);
      pstmt.setString(1, id);
      result = pstmt.executeQuery();
      if (result.next()) {
        String newId = result.getString("id");
        String title = result.getString("title");
        String director = result.getString("director");
        dvd = new DVD(newId, title, new Key(director));
      }
    } catch (SQLException e) {
      error = e;
      message = "Read failed";
    }

    closeResult(result);
    closePrep(pstmt);
    closeConnection(conn);
    checkOK();

    return dvd;
  }

  public void updateDVD(String id, String title) throws DAOException {
    if (findDVD(id) == null)
      throw new DAOException("Id " + id + " was not found");
    Connection conn = null;
    PreparedStatement pstmt = null;
    error = null;

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.updateDVD, id=" + id);
      // Prepare a statement to update a record
      String sql = "UPDATE dvd SET title=? WHERE id=?";
      pstmt = conn.prepareStatement(sql);
      pstmt.setString(1, title);
      pstmt.setString(2, id);
      pstmt.executeUpdate();
    } catch (SQLException e) {
      error = e;
      message = "Update failed";
    }

    closePrep(pstmt);
    closeConnection(conn);
    checkOK();
  }

  public void deleteDVD(String id) throws DAOException {
    if (findDVD(id) == null)
      throw new DAOException("Id " + id + " was not found");
    Connection conn = null;
    PreparedStatement pstmt = null;
    error = null;

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.deleteDVD, id=" + id);
      // Prepare a statement to delete a record
      String sql = "DELETE FROM dvd WHERE id=?";
      pstmt = conn.prepareStatement(sql);
      pstmt.setString(1, id);
      pstmt.executeUpdate();
    } catch (SQLException e) {
      error = e;
      message = "Delete failed";
    }

    closePrep(pstmt);
    closeConnection(conn);
    checkOK();

  }

  public Collection findAll() throws DAOException {
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    error = null;
    Collection c = new ArrayList();

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.getAll");
      // Prepare a statement to delete a record
      String sql = "SELECT * FROM dvd";
      pstmt = conn.prepareStatement(sql);
      rs = pstmt.executeQuery(sql);
      while (rs.next()) {
        String id = rs.getString("id");
        String title = rs.getString("title");
        String director = rs.getString("director");
        DVD dvd = new DVD(id, title, new Key(director));
        c.add(dvd);
      }
    } catch (SQLException e) {
      error = e;
      message = "Getall failed";
    }

    closeResult(rs);
    closePrep(pstmt);
    closeConnection(conn);
    checkOK();
    return c;

  }

  public Collection findAllDVDKeys() throws DAOException {
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    error = null;
    Collection c = new ArrayList();

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.getAll");
      // Prepare a statement to delete a record
      String sql = "SELECT director FROM dvd";
      pstmt = conn.prepareStatement(sql);
      rs = pstmt.executeQuery(sql);
      while (rs.next()) {
        String director = rs.getString("director");
        Key key = new Key(director);
        c.add(key);
      }
    } catch (SQLException e) {
      error = e;
      message = "Getall failed";
    }

    closeResult(rs);
    closePrep(pstmt);
    closeConnection(conn);
    checkOK();
    return c;

  }

  public Collection findAllDVDWithTitle(String findTitle) throws DAOException {
    Connection conn = null;
    PreparedStatement pstmt = null;
    ResultSet rs = null;
    error = null;
    Collection c = new ArrayList();

    try {
      conn = getConnection();
      logger.info("DatabaseDAO.findDVDTitle, title=" + findTitle);
      // Prepare a statement to delete a record
      String sql = "SELECT * FROM dvd";
      pstmt = conn.prepareStatement(sql);
      rs = pstmt.executeQuery(sql);
      while (rs.next()) {
        String id = rs.getString("id");
        String title = rs.getString("title");
        String director = rs.getString("director");
        DVD dvd = new DVD(id, title, new Key(director));
        if (dvd.getTitle().toUpperCase().indexOf(findTitle.toUpperCase()) > -1) {
          c.add(dvd);
        }
      }
    } catch (SQLException e) {
      error = e;
      message = "Find failed";
    }

    closeResult(rs);
    closePrep(pstmt);
    closeConnection(conn);
    checkOK();
    return c;

  }

}