package dk.hansen.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import dk.hansen.transaction.JDBCTransaction;
import dk.hansen.transaction.TransactionIF;

public class DataBase {

  private JDBCTransaction trans;

  private static Logger logger = Logger.getLogger(DataBase.class.getName());

  // Contains en Exception when an error has occured
  protected Exception error;

  // Contains an error message
  protected String message;

  public void setTransaction(TransactionIF trans) {
    this.trans = (JDBCTransaction) trans;
  }

  protected Connection getConnection() throws DAOException {
    if (trans == null) {
      throw new DAOException("The transaction instance is not initialized");
    }
    Connection conn = trans.getConnection();
    if (conn == null) {
      throw new DAOException("The transaction instance did not return a non-null JDBC connection");
    }
    return conn;
  }

  protected void checkOK() throws DAOException {
    if (error != null) {
      throw new DAOException(message, error);
    }
  }

  protected void closeConnection(Connection connection) {
    if (connection != null) {
      try {
        if (connection.getAutoCommit()) {
          connection.close();
          connection = null;
        }
      } catch (SQLException e) {
        if (error == null) {
          error = e;
          message = "Close conn failed";
        }
      }
    }
  }

  protected void closePrep(PreparedStatement prep) {
    if (prep != null) {
      try {
        prep.close();
      } catch (SQLException e) {
        if (error == null) {
          error = e;
          message = "Close prep failed";
        }
      }
    }
  }

  protected void closeResult(ResultSet rs) {
    if (rs != null) {
      try {
        rs.close();
      } catch (SQLException e) {
        if (error == null) {
          error = e;
          message = "Close result failed";
        }
      }
    }
  }

}