package dk.hansen.dao;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.log4j.Logger;

import dk.hansen.transaction.JDBCTransaction;
import dk.hansen.transaction.TransactionIF;

/*
 * This is a factory for the DAO. It uses plain JDBC calls
 */
public class JdbcDAOFactory implements DAOFactoryIF {

  private DataSource dataSource;

  private static Logger logger = Logger.getLogger(JdbcDAOFactory.class
      .getName());

  public JdbcDAOFactory() throws DAOException {
    //    Load the MySQL server JDBC driver
    String driverName = "org.gjt.mm.mysql.Driver";
    try {
      Class.forName(driverName);
    } catch (ClassNotFoundException e) {
      throw new DAOException("JDBC Driver " + driverName
          + " could not be loaded");
    }
    logger.info("JdbcDAOFactory. JDBC driver loaded");
    dataSource = new MyDataSource();
    logger.info("JdbcDAOFactory. Datasource loaded");
  }

  public TransactionIF createTransaction() {
    JDBCTransaction trans = new JDBCTransaction();
    trans.setDataSource(dataSource);
    return trans;
  }

  public DVDDAOIF createDVDDAO() {
    DVDDAO dao = new DVDDAO();
    return dao;
  }

  public DirectorDAOIF createDirectorDAO() {
    DirectorDAO director = new DirectorDAO();
    return director;
  }

  private class MyDataSource implements DataSource {

    public int getLoginTimeout() throws SQLException {
      return 0;
    }

    public void setLoginTimeout(int arg0) throws SQLException {
    }

    public PrintWriter getLogWriter() throws SQLException {
      return null;
    }

    public void setLogWriter(PrintWriter arg0) throws SQLException {
    }

    public Connection getConnection() throws SQLException {
      //    Create a connection to the database
      String serverName = "localhost";
      String mydatabase = "test";
      String url = "jdbc:mysql://" + serverName + "/" + mydatabase;
      String username = "";
      String password = "";
      return DriverManager.getConnection(url, username, password);
    }

    public Connection getConnection(String arg0, String arg1)
        throws SQLException {
      return null;
    }
  }

}