package dk.hansen.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import dk.hansen.dao.bean.DVD;
import dk.hansen.dao.bean.Key;
import dk.hansen.transaction.MyTransaction;
import dk.hansen.transaction.TransactionIF;


public class SimpleDVDDAO implements DVDDAOIF {

  private MyTransaction trans;

   private static HashMap dvds = new HashMap();

   static {
      System.out.println("SimpleDAO created - static code");
      // Create two DVDs for testing
      String id; DVD dvd;
      id = "ID1";
      dvd = new DVD(id, "Troy", null);
      dvds.put(id, dvd);
      id = "ID2";
      dvd = new DVD(id, "Peter Pan", null);
      dvds.put(id, dvd);
   }
      
   public void createDVD(String id, String title, String directorId) throws DAOException {
      DVD dvd = (DVD)dvds.get(id);
      if (dvd != null) throw new DAOException("Id " + id + " is already used");
      dvd = new DVD(id, title, new Key(directorId));
      trans.saveState(dvds);
      dvds.put(id, dvd);
      System.out.println("SimpleDAO created " + id);
   }

   public void updateDVD(String id, String title) throws DAOException {
      DVD dvd = (DVD)dvds.get(id);
      if (dvd == null) throw new DAOException("Id " + id + " was not found");
      trans.saveState(dvds);
      dvd.setTitle(title);
   }

   public void deleteDVD(String id) throws DAOException {
      DVD dvd = (DVD)dvds.get(id);
      if (dvd == null) throw new DAOException("Id " + id + " was not found");
      trans.saveState(dvds);
      dvds.remove(id);
      System.out.println("SimpleDAO deleted " + id);
   }

   public DVD findDVD(String id) throws DAOException {
      DVD dvd = (DVD)dvds.get(id);
      return dvd;
   }

   public Collection findAll() {
     return dvds.values();
  }

   public Collection findAllDVDKeys() {
     return dvds.keySet();
  }

   public Collection findAllDVDWithTitle(String title) {
      Collection hits = new ArrayList(); 
      Collection c = dvds.values();
      for (Iterator it = c.iterator(); it.hasNext();) {
         DVD dvd = (DVD)it.next();
         if (dvd.getTitle().toUpperCase().indexOf(title.toUpperCase()) > -1) {
            hits.add(dvd);
         }
      }
      return hits;
   }

  public void setTransaction(TransactionIF trans) {
    this.trans = (MyTransaction)trans;
  }
}
