package dk.hansen.transaction;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import dk.hansen.dao.DAOException;
import dk.hansen.dao.bean.DVD;

/**
 * This is a simple implementation of transaction handling for a back-end based on a hashmap.
 * When a transaction starts the hashmap is saved.
 * When the transaction rolls back the hash map is restored from the saved hashmap. 
 */
public class MyTransaction implements TransactionIF {

  private Map savedState;
  private Map daoState;
  
  private boolean saveFlag = false;
  
  private static Logger logger = Logger.getLogger(MyTransaction.class
      .getName());
  
  public void begin() throws DAOException {
    logger.info("begin called");
    saveFlag = true;
  }

  public void rollback() throws DAOException {
    logger.info("rollback called");
    daoState.clear();
    daoState.putAll(savedState);
    //printState();
  }

  public void end() throws DAOException {
    logger.info("end called");
    daoState = null;
    savedState = null;
  }

  public void saveState(Map state) {
    if (saveFlag) {
      this.savedState = new HashMap(state);
      this.daoState = state;
      saveFlag = false;
    }
  }

  private void printState() {
    Set keys = savedState.keySet();
    for (Iterator it = keys.iterator(); it.hasNext();) {
      String id = (String)it.next();
      DVD dvd = (DVD)savedState.get(id);
      System.out.println("###Id=" + id + ", title=" + dvd.getTitle());
    }
  }

}
