package dk.hansen.unittest;


import junit.framework.TestCase;
import dk.hansen.TransactionHandler;
import dk.hansen.dao.DAOException;
import dk.hansen.dao.DAOFactoryIF;
import dk.hansen.dao.DVDDAOIF;
import dk.hansen.dto.DVD_DTO;
import dk.hansen.dto.Key_DTO;
import dk.hansen.transaction.TransactionIF;

public class UnitTest1 extends TestCase {

  private DVDDAOIF dao;
  private TransactionIF trans;
  
  protected void setUp() throws Exception {
    // Create a factory class
    String managerName = "dk.hansen.dao.JdbcDAOFactory";
    //String managerName = "dk.hansen.dao.SimpleDAOFactory";
    DAOFactoryIF factory = (DAOFactoryIF) Class.forName(managerName).newInstance();

    // Create a transaction and a DAO
    trans = factory.createTransaction();
    dao = factory.createDVDDAO();
    dao.setTransaction(trans);
    
    
    String id1 = "ID1";
    String id2 = "ID2";
    String id3 = "ID3";
    String id4 = "ID4";

    try {
      dao.deleteDVD(id1);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id2);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id3);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id4);
    } catch (Exception e) {}

    dao.createDVD(id1, "Lord of the Rings (1)", "1");
    dao.createDVD(id2, "The Matrix", "1");

  }

  public void testFind() throws DAOException {
    TransactionHandler handler = new TransactionHandler();
    long key1 = Math.round(Math.random()*1000000);
    Key_DTO key = new Key_DTO(key1+"");
    long key2 = Math.round(Math.random()*1000000);
    Key_DTO directorKey = new Key_DTO(key2+"");
    
    DVD_DTO dvd = new DVD_DTO();
    dvd.setKey(key);
    dvd.setTitle("Ivanhoe");
    dvd.setDirectorKey(directorKey);

    handler.create(dvd);
    
    DVD_DTO dvd2 = handler.findDVD(key.getId());
    assertEquals(dvd2.getTitle(), dvd.getTitle());
    
    handler.delete(dvd); 

    DVD_DTO dvd3 = handler.findDVD(key.getId());
    assertNull(dvd3);
  }
  
  public static void main(String[] args) {
    junit.textui.TestRunner.run(UnitTest1.class);
  }

}